# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.



# The name this table is to be known by, with the format of the mappings in
# the main body of the table, and what all code points missing from this file
# map to.
$Unicode::UCD::SwashInfo{'ToVo'}{'format'} = 's'; # string
$Unicode::UCD::SwashInfo{'ToVo'}{'missing'} = 'R';

return <<'END';
0	A6	Rotated
A7		U
A8		Rotated
A9		U
AA	AD	Rotated
AE		U
AF	B0	Rotated
B1		U
B2	BB	Rotated
BC	BE	U
BF	D6	Rotated
D7		U
D8	F6	Rotated
F7		U
F8	2E9	Rotated
2EA	2EB	U
2EC	377	Rotated
37A	37F	Rotated
384	38A	Rotated
38C		Rotated
38E	3A1	Rotated
3A3	52F	Rotated
531	556	Rotated
559	58A	Rotated
58D	58F	Rotated
591	5C7	Rotated
5D0	5EA	Rotated
5EF	5F4	Rotated
600	61C	Rotated
61E	70D	Rotated
70F	74A	Rotated
74D	7B1	Rotated
7C0	7FA	Rotated
7FD	82D	Rotated
830	83E	Rotated
840	85B	Rotated
85E		Rotated
860	86A	Rotated
8A0	8B4	Rotated
8B6	8C7	Rotated
8D3	983	Rotated
985	98C	Rotated
98F	990	Rotated
993	9A8	Rotated
9AA	9B0	Rotated
9B2		Rotated
9B6	9B9	Rotated
9BC	9C4	Rotated
9C7	9C8	Rotated
9CB	9CE	Rotated
9D7		Rotated
9DC	9DD	Rotated
9DF	9E3	Rotated
9E6	9FE	Rotated
A01	A03	Rotated
A05	A0A	Rotated
A0F	A10	Rotated
A13	A28	Rotated
A2A	A30	Rotated
A32	A33	Rotated
A35	A36	Rotated
A38	A39	Rotated
A3C		Rotated
A3E	A42	Rotated
A47	A48	Rotated
A4B	A4D	Rotated
A51		Rotated
A59	A5C	Rotated
A5E		Rotated
A66	A76	Rotated
A81	A83	Rotated
A85	A8D	Rotated
A8F	A91	Rotated
A93	AA8	Rotated
AAA	AB0	Rotated
AB2	AB3	Rotated
AB5	AB9	Rotated
ABC	AC5	Rotated
AC7	AC9	Rotated
ACB	ACD	Rotated
AD0		Rotated
AE0	AE3	Rotated
AE6	AF1	Rotated
AF9	AFF	Rotated
B01	B03	Rotated
B05	B0C	Rotated
B0F	B10	Rotated
B13	B28	Rotated
B2A	B30	Rotated
B32	B33	Rotated
B35	B39	Rotated
B3C	B44	Rotated
B47	B48	Rotated
B4B	B4D	Rotated
B55	B57	Rotated
B5C	B5D	Rotated
B5F	B63	Rotated
B66	B77	Rotated
B82	B83	Rotated
B85	B8A	Rotated
B8E	B90	Rotated
B92	B95	Rotated
B99	B9A	Rotated
B9C		Rotated
B9E	B9F	Rotated
BA3	BA4	Rotated
BA8	BAA	Rotated
BAE	BB9	Rotated
BBE	BC2	Rotated
BC6	BC8	Rotated
BCA	BCD	Rotated
BD0		Rotated
BD7		Rotated
BE6	BFA	Rotated
C00	C0C	Rotated
C0E	C10	Rotated
C12	C28	Rotated
C2A	C39	Rotated
C3D	C44	Rotated
C46	C48	Rotated
C4A	C4D	Rotated
C55	C56	Rotated
C58	C5A	Rotated
C60	C63	Rotated
C66	C6F	Rotated
C77	C8C	Rotated
C8E	C90	Rotated
C92	CA8	Rotated
CAA	CB3	Rotated
CB5	CB9	Rotated
CBC	CC4	Rotated
CC6	CC8	Rotated
CCA	CCD	Rotated
CD5	CD6	Rotated
CDE		Rotated
CE0	CE3	Rotated
CE6	CEF	Rotated
CF1	CF2	Rotated
D00	D0C	Rotated
D0E	D10	Rotated
D12	D44	Rotated
D46	D48	Rotated
D4A	D4F	Rotated
D54	D63	Rotated
D66	D7F	Rotated
D81	D83	Rotated
D85	D96	Rotated
D9A	DB1	Rotated
DB3	DBB	Rotated
DBD		Rotated
DC0	DC6	Rotated
DCA		Rotated
DCF	DD4	Rotated
DD6		Rotated
DD8	DDF	Rotated
DE6	DEF	Rotated
DF2	DF4	Rotated
E01	E3A	Rotated
E3F	E5B	Rotated
E81	E82	Rotated
E84		Rotated
E86	E8A	Rotated
E8C	EA3	Rotated
EA5		Rotated
EA7	EBD	Rotated
EC0	EC4	Rotated
EC6		Rotated
EC8	ECD	Rotated
ED0	ED9	Rotated
EDC	EDF	Rotated
F00	F47	Rotated
F49	F6C	Rotated
F71	F97	Rotated
F99	FBC	Rotated
FBE	FCC	Rotated
FCE	FDA	Rotated
1000	10C5	Rotated
10C7		Rotated
10CD		Rotated
10D0	10FF	Rotated
1100	11FF	U
1200	1248	Rotated
124A	124D	Rotated
1250	1256	Rotated
1258		Rotated
125A	125D	Rotated
1260	1288	Rotated
128A	128D	Rotated
1290	12B0	Rotated
12B2	12B5	Rotated
12B8	12BE	Rotated
12C0		Rotated
12C2	12C5	Rotated
12C8	12D6	Rotated
12D8	1310	Rotated
1312	1315	Rotated
1318	135A	Rotated
135D	137C	Rotated
1380	1399	Rotated
13A0	13F5	Rotated
13F8	13FD	Rotated
1400		Rotated
1401	167F	U
1680	169C	Rotated
16A0	16F8	Rotated
1700	170C	Rotated
170E	1714	Rotated
1720	1736	Rotated
1740	1753	Rotated
1760	176C	Rotated
176E	1770	Rotated
1772	1773	Rotated
1780	17DD	Rotated
17E0	17E9	Rotated
17F0	17F9	Rotated
1800	180E	Rotated
1810	1819	Rotated
1820	1878	Rotated
1880	18AA	Rotated
18B0	18FF	U
1900	191E	Rotated
1920	192B	Rotated
1930	193B	Rotated
1940		Rotated
1944	196D	Rotated
1970	1974	Rotated
1980	19AB	Rotated
19B0	19C9	Rotated
19D0	19DA	Rotated
19DE	1A1B	Rotated
1A1E	1A5E	Rotated
1A60	1A7C	Rotated
1A7F	1A89	Rotated
1A90	1A99	Rotated
1AA0	1AAD	Rotated
1AB0	1AC0	Rotated
1B00	1B4B	Rotated
1B50	1B7C	Rotated
1B80	1BF3	Rotated
1BFC	1C37	Rotated
1C3B	1C49	Rotated
1C4D	1C88	Rotated
1C90	1CBA	Rotated
1CBD	1CC7	Rotated
1CD0	1CFA	Rotated
1D00	1DF9	Rotated
1DFB	1F15	Rotated
1F18	1F1D	Rotated
1F20	1F45	Rotated
1F48	1F4D	Rotated
1F50	1F57	Rotated
1F59		Rotated
1F5B		Rotated
1F5D		Rotated
1F5F	1F7D	Rotated
1F80	1FB4	Rotated
1FB6	1FC4	Rotated
1FC6	1FD3	Rotated
1FD6	1FDB	Rotated
1FDD	1FEF	Rotated
1FF2	1FF4	Rotated
1FF6	1FFE	Rotated
2000	2015	Rotated
2016		U
2017	201F	Rotated
2020	2021	U
2022	202F	Rotated
2030	2031	U
2032	203A	Rotated
203B	203C	U
203D	2041	Rotated
2042		U
2043	2046	Rotated
2047	2049	U
204A	2050	Rotated
2051		U
2052	2064	Rotated
2065		U
2066	2071	Rotated
2074	208E	Rotated
2090	209C	Rotated
20A0	20BF	Rotated
20D0	20DC	Rotated
20DD	20E0	U
20E1		Rotated
20E2	20E4	U
20E5	20F0	Rotated
2100	2101	U
2102		Rotated
2103	2109	U
210A	210E	Rotated
210F		U
2110	2112	Rotated
2113	2114	U
2115		Rotated
2116	2117	U
2118	211D	Rotated
211E	2123	U
2124		Rotated
2125		U
2126		Rotated
2127		U
2128		Rotated
2129		U
212A	212D	Rotated
212E		U
212F	2134	Rotated
2135	213F	U
2140	2144	Rotated
2145	214A	U
214B		Rotated
214C	214D	U
214E		Rotated
214F	2189	U
218A	218B	Rotated
218C	218F	U
2190	221D	Rotated
221E		U
221F	2233	Rotated
2234	2235	U
2236	22FF	Rotated
2300	2307	U
2308	230B	Rotated
230C	231F	U
2320	2323	Rotated
2324	2328	U
2329	232A	Tr
232B		U
232C	237C	Rotated
237D	239A	U
239B	23BD	Rotated
23BE	23CD	U
23CE		Rotated
23CF		U
23D0		Rotated
23D1	23DB	U
23DC	23E1	Rotated
23E2	2422	U
2423		Rotated
2424	24FF	U
2500	259F	Rotated
25A0	2619	U
261A	261F	Rotated
2620	2767	U
2768	2775	Rotated
2776	2793	U
2794	2B11	Rotated
2B12	2B2F	U
2B30	2B4F	Rotated
2B50	2B59	U
2B5A	2B73	Rotated
2B76	2B95	Rotated
2B97		U
2B98	2BB7	Rotated
2BB8	2BD1	U
2BD2		Rotated
2BD3	2BEB	U
2BEC	2BEF	Rotated
2BF0	2BFF	U
2C00	2C2E	Rotated
2C30	2C5E	Rotated
2C60	2CF3	Rotated
2CF9	2D25	Rotated
2D27		Rotated
2D2D		Rotated
2D30	2D67	Rotated
2D6F	2D70	Rotated
2D7F	2D96	Rotated
2DA0	2DA6	Rotated
2DA8	2DAE	Rotated
2DB0	2DB6	Rotated
2DB8	2DBE	Rotated
2DC0	2DC6	Rotated
2DC8	2DCE	Rotated
2DD0	2DD6	Rotated
2DD8	2DDE	Rotated
2DE0	2E4F	Rotated
2E50	2E51	U
2E52		Rotated
2E80	3000	U
3001	3002	Tu
3003	3007	U
3008	3011	Tr
3012	3013	U
3014	301F	Tr
3020	302F	U
3030		Tr
3031	3040	U
3041		Tu
3042		U
3043		Tu
3044		U
3045		Tu
3046		U
3047		Tu
3048		U
3049		Tu
304A	3062	U
3063		Tu
3064	3082	U
3083		Tu
3084		U
3085		Tu
3086		U
3087		Tu
3088	308D	U
308E		Tu
308F	3094	U
3095	3096	Tu
3097	309A	U
309B	309C	Tu
309D	309F	U
30A0		Tr
30A1		Tu
30A2		U
30A3		Tu
30A4		U
30A5		Tu
30A6		U
30A7		Tu
30A8		U
30A9		Tu
30AA	30C2	U
30C3		Tu
30C4	30E2	U
30E3		Tu
30E4		U
30E5		Tu
30E6		U
30E7		Tu
30E8	30ED	U
30EE		Tu
30EF	30F4	U
30F5	30F6	Tu
30F7	30FB	U
30FC		Tr
30FD	3126	U
3127		Tu
3128	31EF	U
31F0	31FF	Tu
3200	32FE	U
32FF	3357	Tu
3358	337A	U
337B	337F	Tu
3380	A4CF	U
A4D0	A62B	Rotated
A640	A6F7	Rotated
A700	A7BF	Rotated
A7C2	A7CA	Rotated
A7F5	A82C	Rotated
A830	A839	Rotated
A840	A877	Rotated
A880	A8C5	Rotated
A8CE	A8D9	Rotated
A8E0	A953	Rotated
A95F		Rotated
A960	A97F	U
A980	A9CD	Rotated
A9CF	A9D9	Rotated
A9DE	A9FE	Rotated
AA00	AA36	Rotated
AA40	AA4D	Rotated
AA50	AA59	Rotated
AA5C	AAC2	Rotated
AADB	AAF6	Rotated
AB01	AB06	Rotated
AB09	AB0E	Rotated
AB11	AB16	Rotated
AB20	AB26	Rotated
AB28	AB2E	Rotated
AB30	AB6B	Rotated
AB70	ABED	Rotated
ABF0	ABF9	Rotated
AC00	D7FF	U
D800	DFFF	Rotated
E000	FAFF	U
FB00	FB06	Rotated
FB13	FB17	Rotated
FB1D	FB36	Rotated
FB38	FB3C	Rotated
FB3E		Rotated
FB40	FB41	Rotated
FB43	FB44	Rotated
FB46	FBC1	Rotated
FBD3	FD3F	Rotated
FD50	FD8F	Rotated
FD92	FDC7	Rotated
FDF0	FDFD	Rotated
FE00	FE0F	Rotated
FE10	FE1F	U
FE20	FE2F	Rotated
FE30	FE48	U
FE49	FE4F	Rotated
FE50	FE52	Tu
FE53	FE57	U
FE58		Rotated
FE59	FE5E	Tr
FE5F	FE62	U
FE63	FE66	Rotated
FE67	FE6F	U
FE70	FE74	Rotated
FE76	FEFC	Rotated
FEFF		Rotated
FF01		Tu
FF02	FF07	U
FF08	FF09	Tr
FF0A	FF0B	U
FF0C		Tu
FF0D		Rotated
FF0E		Tu
FF0F	FF19	U
FF1A	FF1B	Tr
FF1C	FF1E	Rotated
FF1F		Tu
FF20	FF3A	U
FF3B		Tr
FF3C		U
FF3D		Tr
FF3E		U
FF3F		Tr
FF40	FF5A	U
FF5B	FF60	Tr
FF61	FFBE	Rotated
FFC2	FFC7	Rotated
FFCA	FFCF	Rotated
FFD2	FFD7	Rotated
FFDA	FFDC	Rotated
FFE0	FFE2	U
FFE3		Tr
FFE4	FFE7	U
FFE8	FFEE	Rotated
FFF0	FFF8	U
FFF9	FFFB	Rotated
FFFC	FFFD	U
10000	1000B	Rotated
1000D	10026	Rotated
10028	1003A	Rotated
1003C	1003D	Rotated
1003F	1004D	Rotated
10050	1005D	Rotated
10080	100FA	Rotated
10100	10102	Rotated
10107	10133	Rotated
10137	1018E	Rotated
10190	1019C	Rotated
101A0		Rotated
101D0	101FD	Rotated
10280	1029C	Rotated
102A0	102D0	Rotated
102E0	102FB	Rotated
10300	10323	Rotated
1032D	1034A	Rotated
10350	1037A	Rotated
10380	1039D	Rotated
1039F	103C3	Rotated
103C8	103D5	Rotated
10400	1049D	Rotated
104A0	104A9	Rotated
104B0	104D3	Rotated
104D8	104FB	Rotated
10500	10527	Rotated
10530	10563	Rotated
1056F		Rotated
10600	10736	Rotated
10740	10755	Rotated
10760	10767	Rotated
10800	10805	Rotated
10808		Rotated
1080A	10835	Rotated
10837	10838	Rotated
1083C		Rotated
1083F	10855	Rotated
10857	1089E	Rotated
108A7	108AF	Rotated
108E0	108F2	Rotated
108F4	108F5	Rotated
108FB	1091B	Rotated
1091F	10939	Rotated
1093F		Rotated
10980	1099F	U
109A0	109B7	Rotated
109BC	109CF	Rotated
109D2	10A03	Rotated
10A05	10A06	Rotated
10A0C	10A13	Rotated
10A15	10A17	Rotated
10A19	10A35	Rotated
10A38	10A3A	Rotated
10A3F	10A48	Rotated
10A50	10A58	Rotated
10A60	10A9F	Rotated
10AC0	10AE6	Rotated
10AEB	10AF6	Rotated
10B00	10B35	Rotated
10B39	10B55	Rotated
10B58	10B72	Rotated
10B78	10B91	Rotated
10B99	10B9C	Rotated
10BA9	10BAF	Rotated
10C00	10C48	Rotated
10C80	10CB2	Rotated
10CC0	10CF2	Rotated
10CFA	10D27	Rotated
10D30	10D39	Rotated
10E60	10E7E	Rotated
10E80	10EA9	Rotated
10EAB	10EAD	Rotated
10EB0	10EB1	Rotated
10F00	10F27	Rotated
10F30	10F59	Rotated
10FB0	10FCB	Rotated
10FE0	10FF6	Rotated
11000	1104D	Rotated
11052	1106F	Rotated
1107F	110C1	Rotated
110CD		Rotated
110D0	110E8	Rotated
110F0	110F9	Rotated
11100	11134	Rotated
11136	11147	Rotated
11150	11176	Rotated
11180	111DF	Rotated
111E1	111F4	Rotated
11200	11211	Rotated
11213	1123E	Rotated
11280	11286	Rotated
11288		Rotated
1128A	1128D	Rotated
1128F	1129D	Rotated
1129F	112A9	Rotated
112B0	112EA	Rotated
112F0	112F9	Rotated
11300	11303	Rotated
11305	1130C	Rotated
1130F	11310	Rotated
11313	11328	Rotated
1132A	11330	Rotated
11332	11333	Rotated
11335	11339	Rotated
1133B	11344	Rotated
11347	11348	Rotated
1134B	1134D	Rotated
11350		Rotated
11357		Rotated
1135D	11363	Rotated
11366	1136C	Rotated
11370	11374	Rotated
11400	1145B	Rotated
1145D	11461	Rotated
11480	114C7	Rotated
114D0	114D9	Rotated
11580	115FF	U
11600	11644	Rotated
11650	11659	Rotated
11660	1166C	Rotated
11680	116B8	Rotated
116C0	116C9	Rotated
11700	1171A	Rotated
1171D	1172B	Rotated
11730	1173F	Rotated
11800	1183B	Rotated
118A0	118F2	Rotated
118FF	11906	Rotated
11909		Rotated
1190C	11913	Rotated
11915	11916	Rotated
11918	11935	Rotated
11937	11938	Rotated
1193B	11946	Rotated
11950	11959	Rotated
119A0	119A7	Rotated
119AA	119D7	Rotated
119DA	119E4	Rotated
11A00	11AAF	U
11AC0	11AF8	Rotated
11C00	11C08	Rotated
11C0A	11C36	Rotated
11C38	11C45	Rotated
11C50	11C6C	Rotated
11C70	11C8F	Rotated
11C92	11CA7	Rotated
11CA9	11CB6	Rotated
11D00	11D06	Rotated
11D08	11D09	Rotated
11D0B	11D36	Rotated
11D3A		Rotated
11D3C	11D3D	Rotated
11D3F	11D47	Rotated
11D50	11D59	Rotated
11D60	11D65	Rotated
11D67	11D68	Rotated
11D6A	11D8E	Rotated
11D90	11D91	Rotated
11D93	11D98	Rotated
11DA0	11DA9	Rotated
11EE0	11EF8	Rotated
11FB0		Rotated
11FC0	11FF1	Rotated
11FFF	12399	Rotated
12400	1246E	Rotated
12470	12474	Rotated
12480	12543	Rotated
13000	1343F	U
14400	1467F	U
16800	16A38	Rotated
16A40	16A5E	Rotated
16A60	16A69	Rotated
16A6E	16A6F	Rotated
16AD0	16AED	Rotated
16AF0	16AF5	Rotated
16B00	16B45	Rotated
16B50	16B59	Rotated
16B5B	16B61	Rotated
16B63	16B77	Rotated
16B7D	16B8F	Rotated
16E40	16E9A	Rotated
16F00	16F4A	Rotated
16F4F	16F87	Rotated
16F8F	16F9F	Rotated
16FE0	18D8F	U
1B000	1B2FF	U
1BC00	1BC6A	Rotated
1BC70	1BC7C	Rotated
1BC80	1BC88	Rotated
1BC90	1BC99	Rotated
1BC9C	1BCA3	Rotated
1D000	1D1FF	U
1D200	1D245	Rotated
1D2E0	1D37F	U
1D400	1D454	Rotated
1D456	1D49C	Rotated
1D49E	1D49F	Rotated
1D4A2		Rotated
1D4A5	1D4A6	Rotated
1D4A9	1D4AC	Rotated
1D4AE	1D4B9	Rotated
1D4BB		Rotated
1D4BD	1D4C3	Rotated
1D4C5	1D505	Rotated
1D507	1D50A	Rotated
1D50D	1D514	Rotated
1D516	1D51C	Rotated
1D51E	1D539	Rotated
1D53B	1D53E	Rotated
1D540	1D544	Rotated
1D546		Rotated
1D54A	1D550	Rotated
1D552	1D6A5	Rotated
1D6A8	1D7CB	Rotated
1D7CE	1D7FF	Rotated
1D800	1DAAF	U
1E000	1E006	Rotated
1E008	1E018	Rotated
1E01B	1E021	Rotated
1E023	1E024	Rotated
1E026	1E02A	Rotated
1E100	1E12C	Rotated
1E130	1E13D	Rotated
1E140	1E149	Rotated
1E14E	1E14F	Rotated
1E2C0	1E2F9	Rotated
1E2FF		Rotated
1E800	1E8C4	Rotated
1E8C7	1E8D6	Rotated
1E900	1E94B	Rotated
1E950	1E959	Rotated
1E95E	1E95F	Rotated
1EC71	1ECB4	Rotated
1ED01	1ED3D	Rotated
1EE00	1EE03	Rotated
1EE05	1EE1F	Rotated
1EE21	1EE22	Rotated
1EE24		Rotated
1EE27		Rotated
1EE29	1EE32	Rotated
1EE34	1EE37	Rotated
1EE39		Rotated
1EE3B		Rotated
1EE42		Rotated
1EE47		Rotated
1EE49		Rotated
1EE4B		Rotated
1EE4D	1EE4F	Rotated
1EE51	1EE52	Rotated
1EE54		Rotated
1EE57		Rotated
1EE59		Rotated
1EE5B		Rotated
1EE5D		Rotated
1EE5F		Rotated
1EE61	1EE62	Rotated
1EE64		Rotated
1EE67	1EE6A	Rotated
1EE6C	1EE72	Rotated
1EE74	1EE77	Rotated
1EE79	1EE7C	Rotated
1EE7E		Rotated
1EE80	1EE89	Rotated
1EE8B	1EE9B	Rotated
1EEA1	1EEA3	Rotated
1EEA5	1EEA9	Rotated
1EEAB	1EEBB	Rotated
1EEF0	1EEF1	Rotated
1F000	1F1FF	U
1F200	1F201	Tu
1F202	1F7FF	U
1F800	1F80B	Rotated
1F810	1F847	Rotated
1F850	1F859	Rotated
1F860	1F887	Rotated
1F890	1F8AD	Rotated
1F8B0	1F8B1	Rotated
1F900	1FAFF	U
1FB00	1FB92	Rotated
1FB94	1FBCA	Rotated
1FBF0	1FBF9	Rotated
20000	2FFFD	U
30000	3FFFD	U
E0001		Rotated
E0020	E007F	Rotated
E0100	E01EF	Rotated
F0000	FFFFD	U
100000	10FFFD	U
END
