# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V58
223
224
304
305
329
330
496
497
912
913
944
945
1415
1416
7830
7835
7838
7839
8016
8017
8018
8019
8020
8021
8022
8023
8064
8112
8114
8117
8118
8120
8124
8125
8130
8133
8134
8136
8140
8141
8146
8148
8150
8152
8162
8165
8166
8168
8178
8181
8182
8184
8188
8189
64256
64263
64275
64280
END
