# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 13.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V42
4352
4608
12289
12292
12296
12306
12307
12320
12334
12337
12343
12344
12539
12540
12593
12687
12800
12831
12896
12927
43360
43389
44032
55204
55216
55239
55243
55292
65093
65095
65377
65382
65440
65471
65474
65480
65482
65488
65490
65496
65498
65501
END
